@@ -1,21 +1,26 @@
+"""
+.. include:: ../../docs/applib/settings.md
+"""
+
+
 import re
 import types
 
 from ..parameters import URL_GH, URL_NB, URL_TFDOC
 from ..core.helpers import console, mdEsc
 from .displaysettings import INTERFACE_OPTIONS
-from .helpers import dm
+from .helpers import dm, parseFeatures, transitiveClosure
 
 
 VAR_PATTERN = re.compile(r"\{([^}]+)\}")
 
 WRITING_DEFAULTS = dict(
-    akk=dict(language="Akkadian", direction="ltr",),
-    hbo=dict(language="Hebrew", direction="rtl",),
-    syc=dict(language="Syriac", direction="rtl",),
-    ara=dict(language="Arabic", direction="rtl",),
-    grc=dict(language="Greek", direction="ltr",),
-    cld=dict(language="Aramaic", direction="ltr",),
+    akk=dict(language="akkadian", direction="ltr",),
+    hbo=dict(language="hebrew", direction="rtl",),
+    syc=dict(language="syriac", direction="rtl",),
+    ara=dict(language="arabic", direction="rtl",),
+    grc=dict(language="greek", direction="ltr",),
+    cld=dict(language="aramaic", direction="ltr",),
 )
 WRITING_DEFAULTS[""] = dict(language="", direction="ltr",)
 
@@ -100,7 +105,7 @@ DOC_DEFAULTS = (
     ("docUrl", "{docBase}/{docPage}{docExt}"),
     ("featureBase", "{docBase}/features/<feature>{docExt}"),
     ("featurePage", "home"),
-    ("charUrl", "{tfDoc}/Writing/Transcription/{language}"),
+    ("charUrl", "{tfDoc}/writing/{language}.html"),
     ("charText", "How TF features represent text"),
 )
 
@@ -122,13 +127,12 @@ TYPE_KEYS = set(
     base
     children
     childrenPlain
-    chunkOf
     condense
     features
     featuresBare
     flow
     graphics
-    hide
+    hidden
     label
     level
     lexOcc
@@ -210,7 +214,7 @@ class Check:
                 if v not in allowedValues:
                     allowed = ",".join(sorted(allowedValues))
                     errors.append(f"{k} must be an integer in {allowed}")
-            elif k == "children":
+            elif k in {"children", "xChildren"}:
                 if type(v) is not str and type(v) is not list:
                     errors.append(f"{k} must be a (list of) node types")
                 else:
@@ -218,7 +222,7 @@ class Check:
                     for tp in v:
                         if tp not in nTypes:
                             errors.append(f"{k}: node type {tp} not present")
-            elif k in {"lexOcc", "chunkOf"}:
+            elif k in {"lexOcc"}:
                 if type(v) is not str or v not in nTypes:
                     errors.append(f"{k}: node type {v} not present")
             elif k == "transform":
@@ -232,7 +236,7 @@ class Check:
                 "lineNumbers",
                 "prettyTypes",
                 "queryFeatures",
-                "showChunks",
+                "showHidden",
                 "showGraphics",
                 "standardFeatures",
                 "withNodes",
@@ -295,7 +299,7 @@ class Check:
                 "childrenPlain",
                 "condense",
                 "graphics",
-                "hide",
+                "hidden",
                 "showVerseInTuple",
                 "stretch",
                 "verselike",
@@ -461,7 +465,7 @@ def setAppSpecsApi(app, cfg):
     dKey = "interfaceDefaults"
     interfaceDefaults = {inf[0]: inf[1] for inf in INTERFACE_OPTIONS}
     dSource = cfg.get(dKey, {})
-    specific = {"lineNumbers", "showChunks", "showGraphics"}
+    specific = {"lineNumbers", "showHidden", "showGraphics"}
 
     allowed = {}
     for (k, v) in interfaceDefaults.items():
@@ -469,8 +473,8 @@ def setAppSpecsApi(app, cfg):
             (
                 k == "lineNumbers"
                 and specs["lineNumberFeature"]
-                or k == "showChunks"
-                and specs["isChunkOf"]
+                or k == "showHidden"
+                and specs["isHidden"]
                 or k == "showGraphics"
                 and specs["hasGraphics"]
             )
@@ -556,7 +560,8 @@ def getTypeDefaults(app, cfg, dKey, withApi):
     api = app.api
     F = api.F
     T = api.T
-    otypeRank = api.otypeRank
+    N = api.N
+    otypeRank = N.otypeRank
     slotType = F.otype.slotType
     nTypes = F.otype.all
     structureTypes = T.structureTypes
@@ -569,7 +574,7 @@ def getTypeDefaults(app, cfg, dKey, withApi):
     specs = app.specs
 
     noChildren = set()
-    isChunkOf = {}
+    isHidden = set()
     featuresBare = {}
     features = {}
     lineNumberFeature = {}
@@ -585,6 +590,7 @@ def getTypeDefaults(app, cfg, dKey, withApi):
     givenLevels = {}
     levels = {}
     childType = {}
+    xChildType = {}
     transform = {}
 
     specs["transform"] = transform
@@ -678,9 +684,9 @@ def getTypeDefaults(app, cfg, dKey, withApi):
         if not info.get("childrenPlain", True):
             noChildren.add(nType)
 
-        chunkOf = info.get("chunkOf", None)
-        if chunkOf is not None:
-            isChunkOf[nType] = chunkOf
+        hidden = info.get("hidden", None)
+        if hidden:
+            isHidden.add(nType)
 
         verselike = info.get("verselike", False)
         if verselike:
@@ -699,6 +705,14 @@ def getTypeDefaults(app, cfg, dKey, withApi):
                 childs = set(childs)
             childType[nType] = set(childs or ())
 
+        if "xChildren" in info:
+            xChilds = info["xChildren"] or ()
+            if type(xChilds) is str:
+                xChilds = {xChilds}
+            else:
+                xChilds = set(xChilds)
+            xChildType[nType] = set(xChilds or ())
+
         checker.report()
 
     lexTypes = set(lexMap.values())
@@ -803,15 +817,24 @@ def getTypeDefaults(app, cfg, dKey, withApi):
             container=containerCls, label=labelCls, children=childrenCls,
         )
 
+    descendantType = transitiveClosure(childType, {slotType})
+
+    for (parent, xChildren) in xChildType.items():
+        if parent in descendantType:
+            descendants = descendantType[parent]
+            for xChild in xChildren:
+                descendants.discard(xChild)
+
     specs.update(
         baseTypes=baseTypes if baseTypes else {slotType},
         childType=childType,
-        chunkedTypes=set(isChunkOf.values()),
+        xChildType=xChildType,
         condenseType=condenseType,
+        descendantType=descendantType,
         features=features,
         featuresBare=featuresBare,
         hasGraphics=hasGraphics,
-        isChunkOf=isChunkOf,
+        isHidden=isHidden,
         labels=labels,
         levels=levels,
         levelCls=levelCls,
@@ -827,9 +850,32 @@ def getTypeDefaults(app, cfg, dKey, withApi):
     )
 
 
-def showContext(app, key=None):
+def showContext(app, *keys):
+    """Shows the *context* of the app `tf.applib.app.App.context` in a pretty way.
+
+    The context is the result of computing sensible defaults for the corpus
+    combined with configuration settings in the app's `config.yaml`.
+
+    Parameters
+    ----------
+    keys: iterable of string
+        For each key passed to this function, the information for that key
+        will be displayed. If no keys are passed, all keys will be displayed.
+
+    Returns
+    -------
+    displayed HTML
+        An expandable list of the key-value pair for the requested keys.
+
+    See Also
+    --------
+    tf.applib.app.App.reuse
+    tf.applib.settings: options allowed in `config.yaml`
+    """
+
     EM = "*empty*"
     block = "    "
+    keys = set(keys)
 
     def eScalar(x, level):
         if type(x) is str and "\n" in x:
@@ -877,16 +923,17 @@ def showContext(app, key=None):
             return eDict(x, level)
         return eRest(x, level)
 
-    openRep = "open" if key is not None else ""
+    openRep1 = "open" if len(keys) else ""
+    openRep2 = "open" if len(keys) == 1 else ""
     md = [
-        f"<details {openRep}>"
+        f"<details {openRep1}>"
         f"<summary><b>{(app.appName)}</b> <i>app context</i></summary>\n\n"
     ]
     for (i, (k, v)) in enumerate(sorted(app.specs.items(), key=lambda y: str(y))):
-        if key is not None and key != k:
+        if len(keys) and k not in keys:
             continue
         md.append(
-            f"<details {openRep}>"
+            f"<details {openRep2}>"
             f"<summary>{i + 1}. {k}</summary>\n\n{eData(v, 0)}\n</details>\n"
         )
     md.append("</details>\n")
@@ -935,17 +982,3 @@ def compileFormatCls(app, specs, givenStyles):
         result[fmt] = textCls
 
     specs["formatCls"] = result
-
-
-def parseFeatures(features):
-    bare = []
-    indirect = {}
-    for feat in features.split(" "):
-        if not feat:
-            continue
-        parts = feat.split(":", 1)
-        feat = parts[-1]
-        bare.append(feat)
-        if len(parts) > 1:
-            indirect[feat] = parts[0]
-    return (bare, indirect)
