@@ -1,3 +1,7 @@
+"""
+.. include:: ../../docs/convert/walker.md
+"""
+
 import collections
 import functools
 import re
@@ -16,13 +20,18 @@ class CV(object):
 
     def __init__(self, TF, silent=False):
         self.TF = TF
-        self.wasSilent = self.TF.tm.isSilent()
-        self.TF.tm.setSilent(silent)
+        tmObj = TF.tmObj
+        isSilent = tmObj.isSilent
+        setSilent = tmObj.setSilent
+
+        self.wasSilent = isSilent()
+        setSilent(silent)
 
     def _showWarnings(self):
-        error = self.TF.tm.error
-        info = self.TF.tm.info
-        indent = self.TF.tm.indent
+        tmObj = self.TF.tmObj
+        error = tmObj.error
+        info = tmObj.info
+        indent = tmObj.indent
 
         warnings = self.warnings
         warn = self.warn
@@ -51,9 +60,10 @@ class CV(object):
                     info("use `cv.walk(..., warn=None)` to suppress warnings")
 
     def _showErrors(self):
-        error = self.TF.tm.error
-        info = self.TF.tm.info
-        indent = self.TF.tm.indent
+        tmObj = self.TF.tmObj
+        error = tmObj.error
+        info = tmObj.info
+        indent = tmObj.indent
         forcedStop = self.forcedStop
 
         errors = self.errors
@@ -88,8 +98,92 @@ class CV(object):
         generateTf=True,
         force=False,
     ):
-        info = self.TF.tm.info
-        indent = self.TF.tm.indent
+        """Asks a director function to walk through source data and receives its actions.
+
+        The `director` function should unravel the source.
+        You have to program the `director`, which takes one argument: `cv`.
+        From the `cv` you can use a few standard actions that instruct Text-Fabric
+        to build a graph.
+
+        This function will check whether the metadata makes sense and is minimally
+        complete.
+
+        During node creation the section structure will be watched,
+        and you will be warned if irregularities occur.
+
+        After the creation of the feature data, some extra checks will be performed
+        to see whether the metadata matches the data and vice versa.
+
+        The new feature data will be written to the output directory of the
+        underlying TF object.  In fact, the rules are exactly the same as for
+        `tf.fabric.Fabric.save`.
+
+        Parameters
+        ----------
+        slotType: string
+            The node type that acts as the type of the slots in the data set.
+
+        oText: dict
+            The configuration information to be stored in the `otext` feature:
+
+            * section types
+            * section features
+            * text formats
+
+        generic: dict
+            Metadata that will be written into the header of all generated TF features.
+
+            You can make changes to this later on, dynamically in your director.
+
+        intFeatures: iterable
+            The set of features that have integer values only.
+
+            You can make changes to this later on, dynamically in your director.
+
+        featureMeta: dict of dict
+            For each node or edge feature descriptive metadata can be supplied.
+
+            You can make changes to this later on, dynamically in your director.
+
+        warn: boolean, optional `True`
+            This regulates the response to warnings:
+
+            `True` (default): stop after warnings (as if they are errors);
+
+            `False` continue after warnings but do show them;
+
+            `None` suppress all warnings.
+
+        silent: boolean, optional `None`
+            By this you can suppress informational messages: `silent=True`.
+
+        force: boolean, optional `False`
+            This forces the process to continue after errors.
+            Your TF set might not be valid.
+            Yet this can be useful during testing, when you know
+            that not everything is OK, but you want to check some results.
+            Especially when dealing with large datasets, you might want to test
+            with little pieces. But then you get a kind of non-fatal errors that
+            stand in the way of testing. For those cases: `force=True`.
+
+        generateTf: boolean, optional `True`
+            You can pass `False` here to suppress the actual writing of TF data.
+            In that way you can dry-run the director to check for errors and warnings
+
+        director: function
+            An ordinary function that takes one argument, the `cv` object, and
+            should not deliver anything.
+
+            Writing this function is the main job to do when you want to convert a data source
+            to TF.
+
+            See `tf.convert.walker` for more details.
+        """
+
+        tmObj = self.TF.tmObj
+        info = tmObj.info
+        indent = tmObj.indent
+        setSilent = tmObj.setSilent
 
         indent(level=0, reset=True)
         info("Importing data from walking through the source ...")
@@ -141,15 +235,16 @@ class CV(object):
                 )
 
         self._showWarnings()
-        self.TF.tm.setSilent(self.wasSilent)
+        setSilent(self.wasSilent)
 
         return self.good
 
     def _prepareMeta(self, otext, generic):
         varRe = re.compile(r"\{([^}]+)\}")
 
-        info = self.TF.tm.info
-        indent = self.TF.tm.indent
+        tmObj = self.TF.tmObj
+        info = tmObj.info
+        indent = tmObj.indent
 
         if not self.good and not self.force:
             return
@@ -343,7 +438,9 @@ class CV(object):
         return good
 
     def stop(self, msg):
-        error = self.TF.tm.error
+        tmObj = self.TF.tmObj
+        error = tmObj.error
+
         error(f"Forced stop: {msg}")
         self.good = False
         self.force = False
@@ -546,7 +643,8 @@ class CV(object):
         # If you resume a slot node, it all non slot nodes in the current context
         # will be linked to it.
 
-        info = self.TF.tm.info
+        tmObj = self.TF.tmObj
+        info = tmObj.info
 
         if not self.good:
             return
@@ -606,8 +704,9 @@ class CV(object):
         self._showErrors()
 
     def _removeUnlinked(self):
-        info = self.TF.tm.info
-        indent = self.TF.tm.indent
+        tmObj = self.TF.tmObj
+        info = tmObj.info
+        indent = tmObj.indent
 
         if not self.good and not self.force:
             return
@@ -658,7 +757,8 @@ class CV(object):
             indent(level=1)
 
     def _checkGraph(self):
-        info = self.TF.tm.info
+        tmObj = self.TF.tmObj
+        info = tmObj.info
 
         if not self.good and not self.force:
             return
@@ -689,7 +789,8 @@ class CV(object):
         self._showErrors()
 
     def _checkFeatures(self):
-        info = self.TF.tm.info
+        tmObj = self.TF.tmObj
+        info = tmObj.info
 
         if not self.good and not self.force:
             return
@@ -827,7 +928,8 @@ class CV(object):
         self._showErrors()
 
     def _reorderNodes(self):
-        info = self.TF.tm.info
+        tmObj = self.TF.tmObj
+        info = tmObj.info
 
         if not self.good and not self.force:
             return
@@ -891,8 +993,9 @@ class CV(object):
         return functools.cmp_to_key(before)
 
     def _reassignFeatures(self):
-        info = self.TF.tm.info
-        indent = self.TF.tm.indent
+        tmObj = self.TF.tmObj
+        info = tmObj.info
+        indent = tmObj.indent
 
         if not self.good and not self.force:
             return
