@@ -1,3 +1,7 @@
+"""
+.. include:: ../../docs/core/text.md
+"""
+
 from .data import WARP
 
 DEFAULT_FORMAT = "text-orig-full"
@@ -7,12 +11,34 @@ TYPE_FMT_SEP = "#"
 
 
 class Text(object):
+    """Low level text representation, including section headings.
+
+    In addition to the methods that are directly defined, there are also
+    methods `xxxName()` and `xxxNode()` where `xxx` is whatever the node type of
+    level 1 sections is.
+
+    !!! note "level 1 node types"
+        In the BHSA the `xxx` is `book`, in the DSS it is `scroll`,
+        in Old Babylonian it is `document`, and in Uruk it is `tablet`.
+
+        Here we take the BHSA as example: `bookName()` and `bookNode()`
+
+            T.bookName(node, lang='en')
+            T.bookNode(name, lang='en')
+
+        with node:int the node in question, name:str the name in question,
+        and `lang='en'` the language of the book name.
+    """
+
     def __init__(self, api):
         self.api = api
         C = api.C
         Fs = api.Fs
         TF = api.TF
         self.languages = {}
+        """A dictionary of the languages that are available for book names.
+        """
+
         self.nameFromNode = {}
         self.nodeFromName = {}
         config = api.TF.features[WARP[2]].metaData if WARP[2] in api.TF.features else {}
@@ -70,23 +96,52 @@ class Text(object):
             setattr(self, f"{sec0}Name", self._sec0Name)
             setattr(self, f"{sec0}Node", self._sec0Node)
 
+        self.formats = {}
+        """The text representation formats that have been defined in your dataset.
+        """
+
         self._compileFormats()
         self.good = good
 
-    def _sec0Name(self, n, lang="en"):
-        sec0T = self.sectionTypes[0]
-        fOtype = self.api.F.otype.v
-        refNode = n if fOtype(n) == sec0T else self.api.L.u(n, sec0T)[0]
-        lookup = self.nameFromNode["" if lang not in self.languages else lang]
-        return lookup.get(refNode, f"not a {sec0T} node")
-
-    def _sec0Node(self, name, lang="en"):
-        sec0T = self.sectionTypes[0]
-        return self.nodeFromName["" if lang not in self.languages else lang].get(
-            (sec0T, name), None
-        )
-
     def sectionTuple(self, n, lastSlot=False, fillup=False):
+        """Gives a tuple of nodes that correspond to a sections
+
+        More precisely, we retrieve the sections that contain a
+        reference node, which is either the first slot or the last slot of the node
+        in question.
+
+        Parameters
+        ----------
+        n: integer
+            The node whose containing section to retrieve.
+        lastSlot: boolean, optional `False`
+            Whether the reference node will be the last slot contained by *n*
+            or the first slot.
+        fillup: boolean, optional `False`
+            Whether to fill up the tuple with missing section elements.
+        Returns
+        -------
+        section: tuple of int
+            If *n* is not a section node, a reference node *r* will be taken
+            according to the *lastSlot* parameter.
+
+            If `fillup == False`:
+
+            If *r* is a level 0 section node,
+            *section* is the 1-element tuple `(r,)`.
+
+            If *r* is a level 1 section node,
+            *section* is a 2-element tuple, i.e.
+            the enclosing level 0 section node and *r* itself.
+
+            If *r* is a level 2 section node,
+            *section* is a 3-element tuple, i.e.
+            the enclosing level 0 and  1 section nodes and *r* itself.
+
+            If `fillup == True`, always a complete 3-tuple of a level 0, 1, and 2
+            section nodes is returned.
+        """
+
         sTypes = self.sectionTypes
         lsTypes = len(sTypes)
         if lsTypes == 0:
@@ -143,6 +198,45 @@ class Text(object):
         return (r0, r1, r2)
 
     def sectionFromNode(self, n, lastSlot=False, lang="en", fillup=False):
+        """Gives the full heading of a section node.
+
+        Parameters
+        ----------
+        n: integer
+            The node whose heading to retrieve.
+        lastSlot: boolean, optional `False`
+            Whether the reference node will be the last slot contained by *n*
+            or the first slot.
+        lang: string, optional `en`
+            The language assumed for the section parts,
+            as far as they are language dependent.
+            Must be a 2-letter language code.
+        fillup: boolean, optional `False`
+            Whether to fill up the tuple with missing section elements.
+
+        Returns
+        -------
+        heading: tuple of pairs
+            If *n* is not a section node, a reference node *r* will be taken
+            according to the *lastSlot* parameter.
+
+            It is the tuple of integer/string values for section ancestors
+            of *r* and *r* itself,
+            where the *fillup* parameter plays the same role as in
+            `Text.sectionTuple`.
+
+        Notes
+        -----
+        !!! hint "crossing verse boundaries"
+            Sometimes a sentence or clause in a verse continue into the next verse.
+            In those cases, this function will return different results for
+            `lastSlot=False` and `lastSlot=True`.
+
+        Warnings
+        --------
+        Nodes that lie outside any book, chapter, or verse
+        will get a `None` in the corresponding members of the returned tuple.
+        """
         sTuple = self.sectionTuple(n, lastSlot=lastSlot, fillup=fillup)
         if len(sTuple) == 0:
             return ()
@@ -159,6 +253,30 @@ class Text(object):
         )
 
     def nodeFromSection(self, section, lang="en"):
+        """Given a section tuple, return the node of it.
+
+        Parameters
+        ----------
+        section: string
+            `section` consists of a book name (in language `lang`),
+            and a chapter number and a verse number
+            (both as strings or number depending on the value type of the
+            corresponding feature).
+        lang: string, optional `en`
+            The language assumed for the section parts,
+            as far as they are language dependent.
+            Must be a 2-letter language code.
+
+        Returns
+        -------
+        section node: integer
+            If section labels for all three levels is present,
+            the result is a level 3 node.
+            If the level 3 label has been left out, the result is a level 2 node.
+            If both level 1 and level 2 labels have been left out,
+            the result is a level 1 node.
+        """
+
         sTypes = self.sectionTypes
         if len(sTypes) == 0:
             return None
@@ -172,9 +290,19 @@ class Text(object):
             return sec2.get(sec0node, {}).get(section[1], {}).get(section[2], None)
 
     def structureInfo(self):
+        """Gives a summary of how structure has been configured in the dataset.
+
+        If there are headings that are the same for multiple structural nodes,
+        you'll get a warning here, and you are told how you can get all of those.
+
+        It also shows a short description of all structure-related methods
+        of the `T` API.
+        """
+
         api = self.api
-        info = api.info
-        error = api.error
+        TF = api.TF
+        info = TF.info
+        error = TF.error
         hdMult = self.hdMult
         hdFromNd = self.hdFromNd
         headings = self.headings
@@ -230,8 +358,23 @@ There are {len(hdFromNd)} structural elements in the dataset.
                 error(f"\tand {nMultiple - 10} headings more")
 
     def structure(self, node=None):
+        """Gives the structure of node and everything below it as a tuple.
+
+        Parameters
+        ----------
+        node: integer, optional `None`
+            The node whose structure is asked for.
+            If *node* is None, the complete structure of the whole dataset is returned.
+
+        Returns
+        -------
+        structure: tuple
+            Nested tuple of nodes involved in the structure below a node.
+        """
+
         api = self.api
-        error = api.error
+        TF = api.TF
+        error = TF.error
         F = api.F
         fOtype = F.otype.v
         hdTop = self.hdTop
@@ -253,6 +396,25 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return (node, tuple(self.structure(node=d) for d in self.down(node)))
 
     def structurePretty(self, node=None, fullHeading=False):
+        """Gives the structure of node and everything below it as a string.
+
+        Parameters
+        ----------
+        node: integer, optional `None`
+            The node whose structure is asked for.
+            If *node* is None, the complete structure of the whole dataset is returned.
+        fullHeading: boolean, optional `False`
+            Normally, for each structural element, only its own subheading is added.
+            But if you want to see the full heading, consisting of the headings of a
+            node and all of its parents, pass `True` for this parameter.
+
+        Returns
+        -------
+        structure: string
+            Pretty representation as string with indentations of the structure
+            below a node.
+        """
+
         structure = self.structure(node=node)
         if structure is None:
             return
@@ -274,8 +436,14 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return "\n".join(material)
 
     def top(self):
+        """Gives all top-level structural nodes in the dataset.
+        These are the nodes that are not embedded in a structural node of the same
+        or a higher level.
+        """
+
         api = self.api
-        error = api.error
+        TF = api.TF
+        error = TF.error
         hdTop = self.hdTop
 
         if hdTop is None:
@@ -284,9 +452,30 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return hdTop
 
     def up(self, n):
+        """Gives the parent of a structural node.
+
+        Parameters
+        ----------
+        n: integer
+            The node whose parent to retrieve.
+
+        Returns
+        -------
+        parent: integer
+            The parent is that structural node that whose heading you get from
+            the heading of *n* minus its last element.
+
+        Notes
+        -----
+        !!!hint "Example"
+            The parent of `((book, Genesis), (chapter, 3), (verse, 16))`
+            is the node that has heading `((book, Genesis), (chapter, 3))`.
+        """
+
         api = self.api
         F = api.F
-        error = api.error
+        TF = api.TF
+        error = TF.error
         fOtype = F.otype.v
 
         hdUp = self.hdUp
@@ -303,10 +492,31 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return hdUp.get(n, None)
 
     def down(self, n):
+        """Gives the children of a structural node.
+
+        Parameters
+        ----------
+        n: integer
+            The node whose children to retrieve.
+
+        Returns
+        -------
+        children: tuple of integers
+            The children are those structural nodes whose headings are one
+            longer than the one from *n*.
+
+        Notes
+        -----
+        !!!hint "Example"
+            The children of `((book, Genesis), (chapter, 3))` are the nodes
+            with heading `((book, Genesis), (chapter, 3), (verse, 1))`, etc.
+        """
+
         api = self.api
         F = api.F
         fOtype = F.otype.v
-        error = api.error
+        TF = api.TF
+        error = TF.error
         hdDown = self.hdDown
         if hdDown is None:
             error(f"structure types are not configured", tm=False)
@@ -321,9 +531,34 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return hdDown.get(n, ())
 
     def headingFromNode(self, n):
+        """Gives the full heading of a structural node.
+
+        Parameters
+        ----------
+        n: integer
+            The node whose heading to retrieve.
+
+        Returns
+        -------
+        heading: tuple of pairs
+            It is the tuple of pairs (node type, feature value)
+            for all ancestors of *n*.
+
+        Notes
+        -----
+        !!!hint "Example"
+            E.g., the heading of the verse node corresponding to Genesis 3:16
+            is `((book, Genesis), (chapter, 3), (verse, 16))`.
+
+        !!!hint "Power tip"
+            If you are interested in the complete mapping: it is stored in
+            the dictionary `hdFromNd`.
+        """
+
         api = self.api
         F = api.F
-        error = api.error
+        TF = api.TF
+        error = TF.error
         fOtype = F.otype.v
         hdFromNd = self.hdFromNd
         if hdFromNd is None:
@@ -339,8 +574,30 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return hdFromNd.get(n, None)
 
     def nodeFromHeading(self, head):
+        """Gives the node corresponding to a heading, provided it exists.
+
+        Parameters
+        ----------
+        head: tuple of pairs
+            See the result of `headingFromNode`.
+
+        Returns
+        -------
+        node: int
+            If there is more than one node that corresponds to the heading,
+            only the last one in the corpus will be returned.
+            `hdMult` contains all such cases.
+
+        Notes
+        -----
+        !!!hint "Power tip"
+            If you are interested in the complete mapping: it is stored in
+            the dictionary `ndFromHd`.
+        """
+
         api = self.api
-        error = api.error
+        TF = api.TF
+        error = TF.error
         ndFromHd = self.ndFromHd
         if ndFromHd is None:
             error(f"structure types are not configured", tm=False)
@@ -350,11 +607,72 @@ There are {len(hdFromNd)} structural elements in the dataset.
         return n
 
     def text(self, nodes, fmt=None, descend=None, func=None, explain=False, **kwargs):
+        """Gives the text that corresponds to a bunch of nodes.
+
+        The
+        [banks](https://nbviewer.jupyter.org/github/annotation/banks/blob/master/programs/formats.ipynb)
+        example corpus shows examples.
+
+        Parameters
+        ----------
+
+        nodes: dict
+            *nodes* can be a single node or an arbitrary iterable of nodes
+            of arbitrary types.
+            No attempt will be made to sort the nodes.
+            If you need order, it is better to sort the nodes first.
+
+        fmt: boolean, optional `None`
+            The text-format of the text representation.
+
+            If it is not specified or `None`, each node will be formatted with
+            a node type specific format, defined as *nodeType*`-default`, if it
+            exists.
+
+            If there is no node specific format, the default format
+            `text-orig-full` will be used.
+
+            If `text-orig-full` is not defined, an error message will be issued,
+            and the nodes will be represented by their types and numbers.
+
+            If a value for *fmt* is passed, but it is not a format defined in the
+            *otext.tf* feature, there will be an error message and `None` is returned.
+
+        descend: boolean, optional `None`
+            Whether to descend to constituent nodes.
+
+            If `True`, nodes will be replaced by a sequence of their consituent nodes,
+            which have a type specified by the format chosen, or, if the format does
+            not specify such a type, the node will be replaced
+            by the slots contained in it.
+
+            If `False`, nodes will not be replaced.
+
+            If *descend* is not specified or None,
+            a node will be replaced by its constituent nodes,
+            unless its type is associated with the given format or,
+            if no format is given, by the default format of its type, or,
+            if there is no such format, by its slots.
+
+            !!! caution "no nodes to descend to"
+                If you call `T.text(n, fmt=myfmt)`
+                and `myfmt` is targeted to a node type that is
+                bigger than the node type of `n`,
+                then the so-called descending leads to an empty
+                sequence of nodes and hence to an empty string.
+
+        explain: boolean, optional `False`
+            The logic of this function is subtle.
+            If you call it and the results baffles you, pass `explain=True`
+            and it will explain what it is doing.
+        """
+
         api = self.api
         E = api.E
         F = api.F
         L = api.L
-        error = api.error
+        TF = api.TF
+        error = TF.error
 
         fOtype = F.otype.v
         slotType = F.otype.slotType
@@ -502,6 +820,19 @@ EXPLANATION: T.text() called with parameters:
             error('Text format "{DEFAULT_FORMAT}" not defined in otext.tf', tm=False)
         return "".join(material)
 
+    def _sec0Name(self, n, lang="en"):
+        sec0T = self.sectionTypes[0]
+        fOtype = self.api.F.otype.v
+        refNode = n if fOtype(n) == sec0T else self.api.L.u(n, sec0T)[0]
+        lookup = self.nameFromNode["" if lang not in self.languages else lang]
+        return lookup.get(refNode, f"not a {sec0T} node")
+
+    def _sec0Node(self, name, lang="en"):
+        sec0T = self.sectionTypes[0]
+        return self.nodeFromName["" if lang not in self.languages else lang].get(
+            (sec0T, name), None
+        )
+
     def _compileFormats(self):
         api = self.api
         TF = api.TF
