@@ -1,9 +1,8 @@
-import os
-
-"""Defined the main class.
-
-Also lists the precomputation steps.
 """
+.. include:: ../docs/main/fabric.md
+"""
+
+import os
 
 import collections
 from .parameters import VERSION, NAME, APIREF, LOCATIONS
@@ -29,17 +28,16 @@ from .core.prepare import (
     sections,
     structure,
 )
+from .core.computed import Computed
+from .core.nodefeature import NodeFeature
+from .core.edgefeature import EdgeFeature
+from .core.otypefeature import OtypeFeature
+from .core.oslotsfeature import OslotsFeature
 from .core.api import (
     Api,
-    NodeFeature,
-    EdgeFeature,
-    OtypeFeature,
-    OslotsFeature,
-    Computed,
-    addSortKey,
+    addNodes,
     addOtype,
     addLocality,
-    addRank,
     addText,
     addSearch,
 )
@@ -86,7 +84,7 @@ KIND = dict(__sections__="section", __structure__="structure")
 
 
 class Fabric(object):
-    """Main class of the TF package.
+    """Initialize the core API for a corpus.
 
     Top level management of
 
@@ -94,20 +92,92 @@ class Fabric(object):
     *   loading and saving feature data
     *   precomputing auxiliary data
     *   caching precomputed and compressed data
+
+    Text-Fabric is initialized for a corpus.
+    It will search a set of directories and catalog all `.tf` files it finds there.
+    These are the features you can subsequently load.
+
+    Here `directories` and `subdirectories` are strings with directory names
+    separated by newlines, or iterables of directories.
+
+    Parameters
+    ----------
+    locations: string | iterable of strings, optional
+        The directories specified here are used as base locations
+        in searching for tf feature files.
+        In general, they will not searched directly, but certain subdirectories
+        of them will be searched, specified by the `modules` parameter.
+
+        Defaults:
+
+            ~/Downloads/text-fabric-data
+            ~/text-fabric-data
+            ~/github/text-fabric-data
+
+        So if you have stored your main Text-Fabric dataset in
+        `text-fabric-data` in one of these directories
+        you do not have to pass a location to Fabric.
+
+    modules: string | iterable of strings
+        The directories specified in here are used as sub directories
+        appended to the directories given by the `locations` parameter.
+
+        All `.tf` files (non-recursively) in any `location/module`
+        will be added to the feature set to be loaded in this session.
+        The order in `modules` is important, because if a feature occurs in
+        multiple modules, the last one will be chosen.
+        In this way you can easily override certain features in one module
+        by features in an other module of your choice.
+
+        Default: `['']`
+
+        So if you leave it out, Text-Fabric will just search the paths specified
+        in `locations`.
+
+    silent:
+        If `True` is passed, banners and normal progress messages are suppressed.
+        If `'deep'` is passed, all informational and warning messages are suppressed.
+        Errors still pass through.
+
+    !!! note "otext@ in modules"
+        If modules contain features with a name starting with `otext@`, then the format
+        definitions in these features will be added to the format definitions in the
+        regular `otext` feature (which is a `tf.core.data.WARP` feature).
+        In this way, modules that define new features for text representation,
+        also can add new formats to the Text-API.
+
+    Returns
+    -------
+    object
+        An object from which you can call up all the of methods of the core API.
     """
 
     def __init__(self, locations=None, modules=None, silent=False):
+
         self.silent = silent
-        self.tm = Timestamp()
-        self.tm.setSilent(silent)
+        tmObj = Timestamp()
+        self.tmObj = tmObj
+        setSilent = tmObj.setSilent
+        setSilent(silent)
         self.banner = f"This is {NAME} {VERSION}"
+        """The banner the Text-Fabric.
+
+        Will be shown just after start up, if the silence is not `deep`.
+        """
+
         self.version = VERSION
+        """The version number of the Text-Fabric library.
+        """
+
         (on32, warn, msg) = check32()
+        warning = tmObj.warning
+        info = tmObj.info
+
         if on32:
-            self.tm.info(warn, tm=False)
+            warning(warn, tm=False)
         if msg:
-            self.tm.info(msg, tm=False)
-        self.tm.info(
+            info(msg, tm=False)
+        info(
             f"""{self.banner}
 Api reference : {APIREF}
 """,
@@ -134,14 +204,77 @@ Api reference : {APIREF}
             "\n\t".join(f"{l}/{f}" for f in self.modules) for l in self.locations
         )
         self.featuresRequested = []
+        self.features = {}
+        """Dictionary of all features that TF has found, whether loaded or not.
+
+        Under each feature name is all info about that feature.
+
+        The best use of this is to get the metadata of features:
+
+            TF.features['fff'].metaData
+
+        This works for all features `fff` that have been found,
+        whether the feature is loaded or not.
+
+        If a feature is loaded, you can also use
+
+        `F.fff.meta` of `E.fff.meta` depending on whether `fff` is a node feature
+        or an edge feature.
+
+        !!! caution "Do not print!"
+            If a feature is loaded, its data is also in the feature info.
+            This can be an enormous amount of information, and you can easily
+            overwhelm your notebook if you print it.
+        """
+
         self._makeIndex()
 
     def load(self, features, add=False, silent=None):
+        """Loads features from disk into RAM memory.
+
+        Parameters
+        ----------
+
+        features: string | iterable
+            Either a string containing space separated feature names, or an
+            iterable of feature names.
+            The feature names are just the names of `.tf` files
+            without directory information and without extension.
+        add: boolean, optional `False`
+            The features will be added to the same currently loaded features, managed
+            by the current API.
+            Meant to be able to dynamically load features without reloading lots
+            of features for nothing.
+        silent: boolean, optional `None`
+            If `False`, the features will be loaded rather silently,
+            most messages will be suppressed.
+            Time consuming operations will always be announced,
+            so that you know what Text-Fabric is doing.
+            If `True` is passed, all informational messages will be suppressed.
+            This is handy I you want to load data as part of other methods, on-the-fly.
+
+        Returns
+        -------
+        boolean | object
+            If `add` is `True` nothing is returned. Otherwise,
+            the result is a new `tf.core.api.Api` if the feature could be loaded,
+            else `False`.
+        """
+
+        tmObj = self.tmObj
+        isSilent = tmObj.isSilent
+        setSilent = tmObj.setSilent
+        indent = tmObj.indent
+        info = tmObj.info
+        warning = tmObj.warning
+        error = tmObj.error
+        cache = tmObj.cache
+
         if silent is not None:
-            wasSilent = self.tm.isSilent()
-            self.tm.setSilent(silent)
-        self.tm.indent(level=0, reset=True)
-        self.tm.info("loading features ...")
+            wasSilent = isSilent()
+            setSilent(silent)
+        indent(level=0, reset=True)
+        info("loading features ...")
         self.sectionsOK = True
         self.structureOK = True
         self.good = True
@@ -174,7 +307,7 @@ Api reference : {APIREF}
                     0 < len(self.sectionFeats) <= 3
                 ):
                     if not add:
-                        self.tm.warning(
+                        warning(
                             f"Dataset without sections in {WARP[2]}:"
                             f"no section functions in the T-API"
                         )
@@ -190,7 +323,7 @@ Api reference : {APIREF}
                     self.textFeatures |= set(self.sectionFeatsWithLanguage)
                 if not self.structureTypes or not self.structureFeats:
                     if not add:
-                        self.tm.warning(
+                        warning(
                             f"Dataset without structure sections in {WARP[2]}:"
                             f"no structure functions in the T-API"
                         )
@@ -213,9 +346,9 @@ Api reference : {APIREF}
             for fName in self.featuresRequested:
                 self._loadFeature(fName)
         if not self.good:
-            self.tm.indent(level=0)
-            self.tm.error("Not all features could be loaded/computed")
-            self.tm.cache()
+            indent(level=0)
+            error("Not all features could be loaded/computed")
+            cache()
             result = False
         elif add:
             try:
@@ -230,14 +363,51 @@ Api reference : {APIREF}
                 console(MEM_MSG)
                 result = False
         if silent is not None:
-            self.tm.setSilent(wasSilent)
+            setSilent(wasSilent)
         if not add:
             return result
 
     def explore(self, silent=None, show=True):
+        """Makes categorization of all features in the dataset.
+
+        Parameters
+        ----------
+        silent: boolean, optional `None`
+            If `False` a message containing the total numbers of features
+            is issued.
+        show: boolean, optional `True`
+            If `False`, the resulting dictionary is delivered in `TF.featureSets`;
+            if `True`, the dictionary is returned as function result.
+
+        Returns
+        -------
+        dict | None
+            A dictionary  with keys `nodes`, `edges`, `configs`, `computeds`.
+            Under each key there is the set of feature names in that category.
+            How this dictionary is delivered, depends on the parameter *show*.
+
+        Notes
+        -----
+        !!! explanation "configs"
+            These are config features, with metadata only, no data. E.g. `otext`.
+
+        !!! explanation "computeds"
+            These are blocks of precomputed data, available under the `C` API,
+            see `tf.core.computed.Computeds`.
+
+        The sets do not indicate whether a feature is loaded or not.
+        There are other functions that give you the loaded features:
+        `tf.core.api.Api.Fall` for nodes and `tf.core.api.Api.Eall` for edges.
+        """
+
+        tmObj = self.tmObj
+        isSilent = tmObj.isSilent
+        setSilent = tmObj.setSilent
+        info = tmObj.info
+
         if silent is not None:
-            wasSilent = self.tm.isSilent()
-            self.tm.setSilent(silent)
+            wasSilent = isSilent()
+            setSilent(silent)
         nodes = set()
         edges = set()
         configs = set()
@@ -254,7 +424,7 @@ Api reference : {APIREF}
             else:
                 dest = nodes
             dest.add(fName)
-        self.tm.info(
+        info(
             "Feature overview: {} for nodes; {} for edges; {} configs; {} computed".format(
                 len(nodes), len(edges), len(configs), len(computeds),
             )
@@ -263,7 +433,7 @@ Api reference : {APIREF}
             nodes=nodes, edges=edges, configs=configs, computeds=computeds
         )
         if silent is not None:
-            self.tm.setSilent(wasSilent)
+            setSilent(wasSilent)
         if show:
             return dict(
                 (kind, tuple(sorted(kindSet)))
@@ -273,6 +443,16 @@ Api reference : {APIREF}
             )
 
     def loadAll(self, silent=None):
+        """Load all loadable features.
+
+        Parameters
+        ----------
+        silent: boolean, optional `None`
+            TF is silent if you specified `silent=True` in a preceding
+            `TF=Fabric()` call.
+            But if you did not, you can also pass `silent=True` to this call.
+        """
+
         api = self.load("", silent=silent)
         allFeatures = self.explore(silent=silent or True, show=True)
         loadableFeatures = allFeatures["nodes"] + allFeatures["edges"]
@@ -280,6 +460,27 @@ Api reference : {APIREF}
         return api
 
     def clearCache(self):
+        """Clears the cache of compiled TF data.
+
+        Text-Fabric precomputes data for you, so that it can be loaded faster.
+        If the original data is updated, Text-Fabric detects it,
+        and will recompute that data.
+
+        But there are cases, when the algorithms of Text-Fabric have changed,
+        without any changes in the data, where you might want to clear the cache
+        of precomputed results.
+
+        Calling this function just does it, and it is equivalent with manually removing
+        all `.tfx` files inside the hidden `.tf` directory inside your dataset.
+
+        !!! hint "No need to load"
+            It is not needed to execute a `TF.load()` first.
+
+        See Also
+        --------
+        tf.clean
+        """
+
         for (fName, fObj) in self.features.items():
             fObj.cleanDataBin()
 
@@ -292,18 +493,105 @@ Api reference : {APIREF}
         module=None,
         silent=None,
     ):
+        """Saves newly generated data to disk as TF features, nodes and/or edges.
+
+        If you have collected feature data in dictionaries, keyed by the
+        names of the features, and valued by their feature data,
+        then you can save that data to `.tf` feature files on disk.
+
+        It is this easy to export new data as features:
+        collect the data and metadata of the features and feed it in an orderly way
+        to `TF.save()` and there you go.
+
+        Parameters
+        ----------
+        nodeFeatures: dict of dict
+            The data of a node feature is a dictionary with nodes as keys (integers!)
+            and strings or numbers as (feature) values.
+            This parameter holds all those dictionaries, keyed by feature name.
+
+        edgeFeatures: dict of dict
+            The data of an edge feature is a dictionary with nodes as keys, and sets or
+            dictionaries as values. These sets should be sets of nodes (integers!),
+            and these dictionaries should have nodes as keys and strings or numbers
+            as values.
+            This parameter holds all those dictionaries, keyed by feature name.
+
+        metadata: dict of  dict
+            The meta data for every feature to be saved is a key-value dictionary.
+            This parameter holds all those dictionaries, keyed by feature name.
+
+            !!! explanation "value types"
+                The type of the feature values ('int' or 'str') should be specified
+                under key `valueType`.
+
+            !!! explanation "edge values"
+                If you save an edge feature, and there are values in that edge feature,
+                you have to say so, by specifying `edgeValues=True`
+                in the metadata for that feature.
+
+            !!! explanation "generic metadata"
+                This parameter may also contain fields under the empty name.
+                These fields will be added to all features in `nodeFeatures` and
+                `edgeFeatures`.
+
+            !!! explanation "config features"
+                If you need to write the *config* feature `otext`,
+                which is a metadata-only feature, just
+                add the metadata under key `otext` in this parameter and make sure
+                that `otext` is not a key in `nodeFeatures` nor in
+                `edgeFeatures`.
+                These fields will be written into the separate config feature `otext`,
+                with no data associated.
+
+        location: dict
+            The (meta)data will be written to the very last directory that TF searched
+            when looking for features (this is determined by the
+            `locations` and `modules` parameters in `tf.fabric.Fabric`.
+
+            If both `locations` and `modules` are empty, writing will take place
+            in the current directory.
+
+            But you can override it:
+
+            If you pass `location=something`, TF will save in `something/mod`,
+            where `mod` is the last meber of the `modules` parameter of TF.
+
+        module: dict
+            This is an additional way of overriding the default location
+            where TF saves new features. See the *location* parameter.
+
+            If you pass `module=something`, TF will save in `loc/something`,
+            where `loc` is the last member of the `locations` parameter of TF.
+
+            If you pass `location=path1` and `module=path2`,
+            TF will save in `path1/path2`.
+
+        silent: boolean, optional `None`
+            TF is silent if you specified `silent=True` in a preceding
+            `TF=Fabric()` call.
+            But if you did not, you can also pass `silent=True` to this call.
+        """
+
+        tmObj = self.tmObj
+        isSilent = tmObj.isSilent
+        setSilent = tmObj.setSilent
+        indent = tmObj.indent
+        info = tmObj.info
+        error = tmObj.error
+
         good = True
         if silent is not None:
-            wasSilent = self.tm.isSilent()
-            self.tm.setSilent(silent)
-        self.tm.indent(level=0, reset=True)
+            wasSilent = isSilent()
+            setSilent(silent)
+        indent(level=0, reset=True)
         self._getWriteLoc(location=location, module=module)
         configFeatures = dict(
             f
             for f in metaData.items()
             if f[0] != "" and f[0] not in nodeFeatures and f[0] not in edgeFeatures
         )
-        self.tm.info(
+        info(
             "Exporting {} node and {} edge and {} config features to {}:".format(
                 len(nodeFeatures),
                 len(edgeFeatures),
@@ -324,7 +612,7 @@ Api reference : {APIREF}
         maxNode = None
         slotType = None
         if WARP[0] in nodeFeatures:
-            self.tm.info(f"VALIDATING {WARP[1]} feature")
+            info(f"VALIDATING {WARP[1]} feature")
             otypeData = nodeFeatures[WARP[0]]
             if type(otypeData) is tuple:
                 (otypeData, slotType, maxSlot, maxNode) = otypeData
@@ -333,16 +621,16 @@ Api reference : {APIREF}
                 maxSlot = max(n for n in otypeData if otypeData[n] == slotType)
                 maxNode = max(otypeData)
         if WARP[1] in edgeFeatures:
-            self.tm.info(f"VALIDATING {WARP[1]} feature")
+            info(f"VALIDATING {WARP[1]} feature")
             oslotsData = edgeFeatures[WARP[1]]
             if type(oslotsData) is tuple:
                 (oslotsData, maxSlot, maxNode) = oslotsData
             if maxSlot is None or maxNode is None:
-                self.tm.error(f"ERROR: cannot check validity of {WARP[1]} feature")
+                error(f"ERROR: cannot check validity of {WARP[1]} feature")
                 good = False
             else:
-                self.tm.info(f"maxSlot={maxSlot:>11}")
-                self.tm.info(f"maxNode={maxNode:>11}")
+                info(f"maxSlot={maxSlot:>11}")
+                info(f"maxNode={maxNode:>11}")
                 maxNodeInData = max(oslotsData)
                 minNodeInData = min(oslotsData)
 
@@ -363,17 +651,15 @@ Api reference : {APIREF}
                             unmappedNodes.append(n)
 
                 if mappedSlotNodes:
-                    self.tm.error(f"ERROR: {WARP[1]} maps slot nodes")
-                    self.tm.error(makeExamples(mappedSlotNodes), tm=False)
+                    error(f"ERROR: {WARP[1]} maps slot nodes")
+                    error(makeExamples(mappedSlotNodes), tm=False)
                     good = False
                 if fakeNodes:
-                    self.tm.error(
-                        f"ERROR: {WARP[1]} maps nodes that are not in {WARP[0]}"
-                    )
-                    self.tm.error(makeExamples(fakeNodes), tm=False)
+                    error(f"ERROR: {WARP[1]} maps nodes that are not in {WARP[0]}")
+                    error(makeExamples(fakeNodes), tm=False)
                     good = False
                 if unmappedNodes:
-                    self.tm.error(f"ERROR: {WARP[1]} fails to map nodes:")
+                    error(f"ERROR: {WARP[1]} fails to map nodes:")
                     unmappedByType = {}
                     for n in unmappedNodes:
                         unmappedByType.setdefault(
@@ -382,13 +668,11 @@ Api reference : {APIREF}
                     for (nType, nodes) in sorted(
                         unmappedByType.items(), key=lambda x: (-len(x[1]), x[0]),
                     ):
-                        self.tm.error(
-                            f"--- unmapped {nType:<10} : {makeExamples(nodes)}"
-                        )
+                        error(f"--- unmapped {nType:<10} : {makeExamples(nodes)}")
                     good = False
 
             if good:
-                self.tm.info(f"OK: {WARP[1]} is valid")
+                info(f"OK: {WARP[1]} is valid")
 
         for (fName, data, isEdge, isConfig) in todo:
             edgeValues = False
@@ -401,7 +685,7 @@ Api reference : {APIREF}
                 del fMeta["edgeValues"]
             fObj = Data(
                 f"{self.writeDir}/{fName}.tf",
-                self.tm,
+                self.tmObj,
                 data=data,
                 metaData=fMeta,
                 isEdge=isEdge,
@@ -413,8 +697,8 @@ Api reference : {APIREF}
                 total[tag] += 1
             else:
                 failed[tag] += 1
-        self.tm.indent(level=0)
-        self.tm.info(
+        indent(level=0)
+        info(
             f"""Exported {total["node"]} node features"""
             f""" and {total["edge"]} edge features"""
             f""" and {total["config"]} config features"""
@@ -422,25 +706,89 @@ Api reference : {APIREF}
         )
         if len(failed):
             for (tag, nf) in sorted(failed.items()):
-                self.tm.error(f"Failed to export {nf} {tag} features")
+                error(f"Failed to export {nf} {tag} features")
             good = False
 
         if silent is not None:
-            self.tm.setSilent(wasSilent)
+            setSilent(wasSilent)
         return good
 
     def exportMQL(self, mqlName, mqlDir):
-        self.tm.indent(level=0, reset=True)
+        """Exports the complete TF dataset into single MQL database.
+
+        Parameters
+        ----------
+        dirName: string
+        dbName: string
+
+        Returns
+        -------
+        None
+            The exported data will be written to file *dirName*`/`*dbName.mql*.
+            If `dirName` starts with `~`, the `~` will be expanded to your
+            home directory.
+            Likewise, `..` will be expanded to the parent of the current directory,
+            and `.` to the current directory, both only at the start of `dirName`.
+
+        See Also
+        --------
+        tf.convert.mql
+        """
+
+        tmObj = self.tmObj
+        indent = tmObj.indent
+
+        indent(level=0, reset=True)
         mqlDir = expandDir(self, mqlDir)
 
         mqlNameClean = cleanName(mqlName)
-        mql = MQL(mqlDir, mqlNameClean, self.features, self.tm)
+        mql = MQL(mqlDir, mqlNameClean, self.features, self.tmObj)
         mql.write()
 
     def importMQL(self, mqlFile, slotType=None, otext=None, meta=None):
-        self.tm.indent(level=0, reset=True)
+        """Converts an MQL database dump to a Text-Fabric dataset.
+
+        !!! hint "Destination directory"
+            It is recommended to call this `importMQL` on a TF instance called with
+
+                TF = Fabric(locations=targetDir)
+
+            Then the resulting features will be written in the targetDir.
+            In fact, the rules are exactly the same as for `save()`.
+
+        Parameters
+        ----------
+        slotType: string
+            You have to tell which object type in the MQL file acts as the slot type,
+            because TF cannot see that on its own.
+
+        otext: dict
+            You can pass the information about sections and text formats as
+            the parameter `otext`. This info will end up in the `otext.tf` feature.
+            Pass it as a dictionary of keys and values, like so:
+
+                otext = {
+                    'fmt:text-trans-plain': '{glyphs}{trailer}',
+                    'sectionFeatures': 'book,chapter,verse',
+                }
+
+        meta: dict
+            Likewise, you can add a dictionary of keys and values that will added to
+            the metadata of all features. Handy to add provenance data here:
+
+                meta = dict(
+                    dataset='DLC',
+                    datasetName='Digital Language Corpus',
+                    author="That 's me",
+                )
+        """
+
+        tmObj = self.tmObj
+        indent = tmObj.indent
+
+        indent(level=0, reset=True)
         (good, nodeFeatures, edgeFeatures, metaData) = tfFromMql(
-            mqlFile, self.tm, slotType=slotType, otext=otext, meta=meta
+            mqlFile, self.tmObj, slotType=slotType, otext=otext, meta=meta
         )
         if good:
             self.save(
@@ -450,10 +798,15 @@ Api reference : {APIREF}
     def _loadFeature(self, fName, optional=False):
         if not self.good:
             return False
-        silent = self.tm.isSilent()
+
+        tmObj = self.tmObj
+        isSilent = tmObj.isSilent
+        error = tmObj.error
+
+        silent = isSilent()
         if fName not in self.features:
             if not optional:
-                self.tm.error(f'Feature "{fName}" not available in\n{self.locationRep}')
+                error(f'Feature "{fName}" not available in\n{self.locationRep}')
                 self.good = False
         else:
             # if not self.features[fName].load(silent=silent or (fName not in self.featuresRequested)):
@@ -461,6 +814,10 @@ Api reference : {APIREF}
                 self.good = False
 
     def _makeIndex(self):
+        tmObj = self.tmObj
+        info = tmObj.info
+        warning = tmObj.warning
+
         self.features = {}
         self.featuresIgnored = {}
         tfFiles = {}
@@ -481,9 +838,9 @@ Api reference : {APIREF}
             for featurePath in sorted(set(featurePaths[0:-1])):
                 if featurePath != chosenFPath:
                     self.featuresIgnored.setdefault(fName, []).append(featurePath)
-            self.features[fName] = Data(chosenFPath, self.tm)
+            self.features[fName] = Data(chosenFPath, self.tmObj)
         self._getWriteLoc()
-        self.tm.info(
+        info(
             "{} features found and {} ignored".format(
                 len(tfFiles), sum(len(x) for x in self.featuresIgnored.values()),
             ),
@@ -494,19 +851,20 @@ Api reference : {APIREF}
         for fName in WARP:
             if fName not in self.features:
                 if fName == WARP[2]:
-                    self.tm.info(
+                    info(
                         (
                             f'Warp feature "{WARP[2]}" not found. Working without Text-API\n'
                         )
                     )
                     self.features[WARP[2]] = Data(
-                        f"{WARP[2]}.tf", self.tm, isConfig=True, metaData=WARP2_DEFAULT,
+                        f"{WARP[2]}.tf",
+                        self.tmObj,
+                        isConfig=True,
+                        metaData=WARP2_DEFAULT,
                     )
                     self.features[WARP[2]].dataLoaded = True
                 else:
-                    self.tm.info(
-                        f'Warp feature "{fName}" not found in\n{self.locationRep}'
-                    )
+                    info(f'Warp feature "{fName}" not found in\n{self.locationRep}')
                     good = False
             elif fName == WARP[2]:
                 self._loadFeature(fName, optional=True)
@@ -525,7 +883,7 @@ Api reference : {APIREF}
                 dependencies = dependencies + sFeats
             for dep in dependencies:
                 if dep not in self.features:
-                    self.tm.info(
+                    warning(
                         f'Missing dependency for computed data feature "{fName}": "{dep}"'
                     )
                     thisGood = False
@@ -533,7 +891,7 @@ Api reference : {APIREF}
                 good = False
             self.features[fName] = Data(
                 f"{self.warpDir}/{fName}.x",
-                self.tm,
+                self.tmObj,
                 method=method,
                 dependencies=[self.features.get(dep, None) for dep in dependencies],
             )
@@ -576,7 +934,12 @@ Api reference : {APIREF}
         if not self.good:
             return None
 
-        silent = self.tm.isSilent()
+        tmObj = self.tmObj
+        isSilent = tmObj.isSilent
+        indent = tmObj.indent
+        info = tmObj.info
+
+        silent = isSilent()
         api = Api(self)
 
         w0info = self.features[WARP[0]]
@@ -626,14 +989,13 @@ Api reference : {APIREF}
                             if hasattr(api.F, fName):
                                 delattr(api.F, fName)
                         fObj.unload()
-        addSortKey(api)
         addOtype(api)
+        addNodes(api)
         addLocality(api)
-        addRank(api)
         addText(api)
         addSearch(api, silent)
-        self.tm.indent(level=0)
-        self.tm.info("All features loaded/computed - for details use loadLog()")
+        indent(level=0)
+        info("All features loaded/computed - for details use loadLog()")
         self.api = api
         return api
 
@@ -641,6 +1003,9 @@ Api reference : {APIREF}
         if not self.good:
             return None
         api = self.api
+        tmObj = self.tmObj
+        indent = tmObj.indent
+        info = tmObj.info
 
         requestedSet = set(self.featuresRequested)
 
@@ -677,5 +1042,5 @@ Api reference : {APIREF}
                             if hasattr(api.F, fName):
                                 delattr(api.F, fName)
                         fObj.unload()
-        self.tm.indent(level=0)
-        self.tm.info("All additional features loaded - for details use loadLog()")
+        indent(level=0)
+        info("All additional features loaded - for details use loadLog()")
