@@ -5,7 +5,7 @@ import sys
 from zipfile import ZIP_DEFLATED
 
 
-VERSION = '8.0.4'
+VERSION = '8.2.2'
 """Program version.
 
 This value is under control of the update process, as run by
@@ -31,10 +31,12 @@ Instead, TF will produce freshly compressed data files.
 The compressed data files are stored in a directory `.tf/{PVN}/` next
 to the original `tf`  files, where `{PVN}` is the package version number.
 
-See also `tf.clean`.
+See Also
+--------
+tf.clean
 """
 
-API_VERSION = 1
+API_VERSION = 2
 """TF API version.
 
 Text-Fabric offers and API to TF apps.
@@ -131,9 +133,12 @@ DOI_URL_PREFIX = "https://doi.org"
 DOI_TF = "10.5281/zenodo.592193"
 """DOI of an archived copy of this repo at Zenodo."""
 
-APIREF = f"https://{ORG}.github.io/{REPO}/Api/Fabric/"
+APIREF = f"https://{ORG}.github.io/{REPO}/cheatsheet.html"
 """Link to the Api docs of Text-Fabric."""
 
+SEARCHREF = f"https://{ORG}.github.io/{REPO}/about/searchusage.html"
+"""Link to the Search docs of Text-Fabric."""
+
 APP_URL = f"{URL_GH}/{ORG}"
 """Url of the GitHub location that contains all the TF apps."""
 
@@ -165,9 +170,7 @@ TEMP_DIR = "_temp"
     Take care that these directories are ignored by git operations.
     Put a line
 
-    ```
-    _temp/
-    ```
+        _temp/
 
     in the `.gitignore` file.
 """
