@@ -64,15 +64,14 @@ opStripRe = re.compile(opStripPat)
 quLineRe = re.compile(quLinePat)
 relRe = re.compile(relPat)
 reRe = re.compile(rePat)
-# whiteRe = re.compile(r'^\s*$')
 whiteRe = re.compile(r"^\s*(%|$)")
 
 reTp = type(reRe)
 
 
 def syntax(searchExe):
-    error = searchExe.api.error
-    msgCache = searchExe.msgCache
+    error = searchExe.api.TF.error
+    _msgCache = searchExe._msgCache
     searchExe.good = True
     searchExe.badSyntax = []
     searchExe.searchLines = searchExe.searchTemplate.split("\n")
@@ -81,12 +80,12 @@ def syntax(searchExe):
     _tokenize(searchExe)
 
     if not searchExe.good:
-        searchExe.showOuterTemplate(msgCache)
+        searchExe.showOuterTemplate(_msgCache)
         for (i, line) in enumerate(searchExe.searchLines):
-            error(f"{i + offset:>2} {line}", tm=False, cache=msgCache)
+            error(f"{i + offset:>2} {line}", tm=False, cache=_msgCache)
         for (ln, eline) in searchExe.badSyntax:
             txt = eline if ln is None else f"line {ln + offset}: {eline}"
-            error(txt, tm=False, cache=msgCache)
+            error(txt, tm=False, cache=_msgCache)
 
 
 def _tokenize(searchExe):
